@@ -1,35 +1,32 @@
-/* @(#)sector.c	1.13 04/03/01 Copyright 2001-2004 J. Schilling */
+/* @(#)sector.c	1.17 09/07/10 Copyright 2001-2009 J. Schilling */
+#include <schily/mconfig.h>
 #ifndef lint
-static	char sccsid[] =
-	"@(#)sector.c	1.13 04/03/01 Copyright 2001-2004 J. Schilling";
+static	UConst char sccsid[] =
+	"@(#)sector.c	1.17 09/07/10 Copyright 2001-2009 J. Schilling";
 #endif
 /*
  *	Functions needed to use libedc_ecc from cdrecord
  *
- *	Copyright (c) 2001-2004 J. Schilling
+ *	Copyright (c) 2001-2009 J. Schilling
  */
 /*
- * This program is free software; you can redistribute it and/or modify
- * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2, or (at your option)
- * any later version.
+ * The contents of this file are subject to the terms of the
+ * Common Development and Distribution License, Version 1.0 only
+ * (the "License").  You may not use this file except in compliance
+ * with the License.
  *
- * This program is distributed in the hope that it will be useful,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
+ * See the file CDDL.Schily.txt in this distribution for details.
  *
- * You should have received a copy of the GNU General Public License along with
- * this program; see the file COPYING.  If not, write to the Free Software
- * Foundation, 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+ * When distributing Covered Code, include this CDDL HEADER in each
+ * file and include the License file CDDL.Schily.txt from this distribution.
  */
 
-#include <mconfig.h>
-#include <stdio.h>
-#include <standard.h>
-#include <utypes.h>
-#include <timedefs.h>
-#include <schily.h>
+#include <schily/mconfig.h>
+#include <schily/stdio.h>
+#include <schily/standard.h>
+#include <schily/utypes.h>
+#include <schily/time.h>
+#include <schily/schily.h>
 
 #include "cdrecord.h"
 #include "movesect.h"
