@@ -1,44 +1,48 @@
-/* @(#)cmpbytes.c	1.14 02/02/28 Copyright 1988 J. Schilling */
+/* @(#)cmpbytes.c	1.22 10/05/24 Copyright 1988, 1995-2010 J. Schilling */
+#include <schily/mconfig.h>
 #ifndef lint
-static	char sccsid[] =
-	"@(#)cmpbytes.c	1.14 02/02/28 Copyright 1988 J. Schilling";
+static	UConst char sccsid[] =
+	"@(#)cmpbytes.c	1.22 10/05/24 Copyright 1988, 1995-2010 J. Schilling";
 #endif  /* lint */
 /*
  *	compare data
+ *	Return the index of the first differing character
  *
- *	Copyright (c) 1988 J. Schilling
+ *	Copyright (c) 1988, 1995-2010 J. Schilling
  */
 /*
- * This program is free software; you can redistribute it and/or modify
- * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2, or (at your option)
- * any later version.
+ * The contents of this file are subject to the terms of the
+ * Common Development and Distribution License, Version 1.0 only
+ * (the "License").  You may not use this file except in compliance
+ * with the License.
  *
- * This program is distributed in the hope that it will be useful,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
+ * See the file CDDL.Schily.txt in this distribution for details.
  *
- * You should have received a copy of the GNU General Public License
- * along with this program; see the file COPYING.  If not, write to
- * the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
+ * When distributing Covered Code, include this CDDL HEADER in each
+ * file and include the License file CDDL.Schily.txt from this distribution.
  */
 
-#include <standard.h>
-#include <align.h>
-#include <schily.h>
+#include <schily/standard.h>
+#include <schily/align.h>
+#include <schily/types.h>
+#include <schily/schily.h>
 
-#define	DO8(a)	a;a;a;a;a;a;a;a;
+#define	DO8(a)	a; a; a; a; a; a; a; a;
 
-int cmpbytes(fromp, top, cnt)
+/*
+ * Return the index of the first differing character
+ * This interface is not compatible to memcmp()
+ */
+EXPORT ssize_t
+cmpbytes(fromp, top, cnt)
 	const void	*fromp;
 	const void	*top;
-	int		cnt;
+	ssize_t		cnt;
 {
 	register const char	*from	= (char *)fromp;
 	register const char	*to	= (char *)top;
-	register int		n;
-	register int		i;
+	register ssize_t	n;
+	register ssize_t	i;
 
 	/*
 	 * If we change cnt to be unsigned, check for == instead of <=
@@ -49,22 +53,22 @@ int cmpbytes(fromp, top, cnt)
 	/*
 	 * Compare byte-wise until properly aligned for a long pointer.
 	 */
-	i = sizeof(long) - 1;
+	i = sizeof (long) - 1;
 	while (--n >= 0 && --i >= 0 && !l2aligned(from, to)) {
 		if (*to++ != *from++)
 			goto cdiff;
 	}
 	n++;
 
-	if (n >= (int)(8 * sizeof(long))) {
+	if (n >= (ssize_t)(8 * sizeof (long))) {
 		if (l2aligned(from, to)) {
 			register const long *froml = (const long *)from;
 			register const long *tol   = (const long *)to;
-			register int rem = n % (8 * sizeof (long));
+			register ssize_t rem = n % (8 * sizeof (long));
 
 			n /= (8 * sizeof (long));
 			do {
-				DO8 (
+				DO8(
 					if (*tol++ != *froml++)
 						break;
 				);
@@ -85,7 +89,7 @@ int cmpbytes(fromp, top, cnt)
 		if (n >= 8) {
 			n -= 8;
 			do {
-				DO8 (
+				DO8(
 					if (*to++ != *from++)
 						goto cdiff;
 				);
@@ -104,11 +108,11 @@ int cmpbytes(fromp, top, cnt)
 	} while (--n > 0);
 	return (cnt);
 ldiff:
-	n = sizeof(long);
+	n = sizeof (long);
 	do {
 		if (*to++ != *from++)
 			goto cdiff;
 	} while (--n > 0);
 cdiff:
-	return (--from - (char *)fromp);
+	return ((ssize_t)(--from - (char *)fromp));
 }
