@@ -1,4 +1,4 @@
-/* @(#)bswap.h	1.2 03/03/25 Copyright 2002 J. Schilling */
+/* @(#)bswap.h	1.3 06/09/13 Copyright 2002 J. Schilling */
 #ifndef _BSWAP_H
 #define	_BSWAP_H
 
@@ -15,9 +15,8 @@
  */
 /*
  * This program is free software; you can redistribute it and/or modify
- * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2, or (at your option)
- * any later version.
+ * it under the terms of the GNU General Public License version 2
+ * as published by the Free Software Foundation.
  *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -29,9 +28,9 @@
  * Foundation, 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  */
 
-#include <mconfig.h>
-#include <btorder.h>
-#include <intcvt.h>
+#include <schily/mconfig.h>
+#include <schily/btorder.h>
+#include <schily/intcvt.h>
 
 #if defined(WORDS_BIGENDIAN)
 
