@@ -1,7 +1,8 @@
-/* @(#)isovfy.c	1.23 04/09/08 joerg */
+/* @(#)isovfy.c	1.43 10/05/24 joerg */
+#include <schily/mconfig.h>
 #ifndef lint
-static	char sccsid[] =
-	"@(#)isovfy.c	1.23 04/09/08 joerg";
+static	UConst char sccsid[] =
+	"@(#)isovfy.c	1.43 10/05/24 joerg";
 #endif
 /*
  * File isovfy.c - verify consistency of iso9660 filesystem.
@@ -10,36 +11,38 @@ static	char sccsid[] =
  * Written by Eric Youngdale (1993).
  *
  * Copyright 1993 Yggdrasil Computing, Incorporated
- * Copyright (c) 1999-2004 J. Schilling
+ * Copyright (c) 1999-2010 J. Schilling
  *
  * This program is free software; you can redistribute it and/or modify
- * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2, or (at your option)
- * any later version.
+ * it under the terms of the GNU General Public License version 2
+ * as published by the Free Software Foundation.
  *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  * GNU General Public License for more details.
  *
- * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+ * You should have received a copy of the GNU General Public License along with
+ * this program; see the file COPYING.  If not, write to the Free Software
+ * Foundation, 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  */
 
-#include <mconfig.h>
-#include <stdxlib.h>
-#include <unixstd.h>
-#include <strdefs.h>
-#include <utypes.h>
+#include <schily/mconfig.h>
+#include <schily/stdlib.h>
+#include <schily/unistd.h>
+#include <schily/string.h>
+#include <schily/utypes.h>
 
-#include <stdio.h>
-#include <standard.h>
-#include <signal.h>
-#include <schily.h>
+#include <schily/stdio.h>
+#include <schily/standard.h>
+#include <schily/signal.h>
+#include <schily/schily.h>
 
+#include "../iso9660.h"
+#include "../rock.h"
 #include "../scsi.h"
-#include "../../cdrecord/defaults.h"
+#include "cdrdeflt.h"
+#include "../../cdrecord/version.h"
 
 /*
  * XXX JS: Some structures have odd lengths!
@@ -71,69 +74,22 @@ int blocksize;
 
 #define	PAGE	sizeof (buffer)
 
-#define	ISODCL(from, to)	(to - from + 1)
-
-struct iso_primary_descriptor {
-	unsigned char type			[ISODCL(1,   1)]; /* 711 */
-	unsigned char id			[ISODCL(2,   6)];
-	unsigned char version			[ISODCL(7,   7)]; /* 711 */
-	unsigned char unused1			[ISODCL(8,   8)];
-	unsigned char system_id			[ISODCL(9,   40)]; /* aunsigned chars */
-	unsigned char volume_id			[ISODCL(41,  72)]; /* dunsigned chars */
-	unsigned char unused2			[ISODCL(73,  80)];
-	unsigned char volume_space_size		[ISODCL(81,  88)]; /* 733 */
-	unsigned char unused3			[ISODCL(89,  120)];
-	unsigned char volume_set_size		[ISODCL(121, 124)]; /* 723 */
-	unsigned char volume_sequence_number	[ISODCL(125, 128)]; /* 723 */
-	unsigned char logical_block_size	[ISODCL(129, 132)]; /* 723 */
-	unsigned char path_table_size		[ISODCL(133, 140)]; /* 733 */
-	unsigned char type_l_path_table		[ISODCL(141, 144)]; /* 731 */
-	unsigned char opt_type_l_path_table	[ISODCL(145, 148)]; /* 731 */
-	unsigned char type_m_path_table		[ISODCL(149, 152)]; /* 732 */
-	unsigned char opt_type_m_path_table	[ISODCL(153, 156)]; /* 732 */
-	unsigned char root_directory_record	[ISODCL(157, 190)]; /* 9.1 */
-	unsigned char volume_set_id		[ISODCL(191, 318)]; /* dunsigned chars */
-	unsigned char publisher_id		[ISODCL(319, 446)]; /* achars */
-	unsigned char preparer_id		[ISODCL(447, 574)]; /* achars */
-	unsigned char application_id		[ISODCL(575, 702)]; /* achars */
-	unsigned char copyright_file_id		[ISODCL(703, 739)]; /* 7.5 dchars */
-	unsigned char abstract_file_id		[ISODCL(740, 776)]; /* 7.5 dchars */
-	unsigned char bibliographic_file_id	[ISODCL(777, 813)]; /* 7.5 dchars */
-	unsigned char creation_date		[ISODCL(814, 830)]; /* 8.4.26.1 */
-	unsigned char modification_date		[ISODCL(831, 847)]; /* 8.4.26.1 */
-	unsigned char expiration_date		[ISODCL(848, 864)]; /* 8.4.26.1 */
-	unsigned char effective_date		[ISODCL(865, 881)]; /* 8.4.26.1 */
-	unsigned char file_structure_version	[ISODCL(882, 882)]; /* 711 */
-	unsigned char unused4			[ISODCL(883, 883)];
-	unsigned char application_data		[ISODCL(884, 1395)];
-	unsigned char unused5			[ISODCL(1396, 2048)];
-};
-
-struct iso_directory_record {
-	unsigned char length			[ISODCL(1, 1)]; /* 711 */
-	unsigned char ext_attr_length		[ISODCL(2, 2)]; /* 711 */
-	unsigned char extent			[ISODCL(3, 10)]; /* 733 */
-	unsigned char size			[ISODCL(11, 18)]; /* 733 */
-	unsigned char date			[ISODCL(19, 25)]; /* 7 by 711 */
-	unsigned char flags			[ISODCL(26, 26)];
-	unsigned char file_unit_size		[ISODCL(27, 27)]; /* 711 */
-	unsigned char interleave		[ISODCL(28, 28)]; /* 711 */
-	unsigned char volume_sequence_number	[ISODCL(29, 32)]; /* 723 */
-	unsigned char name_len			[ISODCL(33, 33)]; /* 711 */
-	unsigned char name			[38];
-};
-
 LOCAL int	isonum_721	__PR((char * p));
 LOCAL int	isonum_723	__PR((char * p));
 LOCAL int	isonum_711	__PR((char * p));
 LOCAL int	isonum_731	__PR((char * p));
+#if 0
 LOCAL int	isonum_722	__PR((char * p));
+#endif
 LOCAL int	isonum_732	__PR((char * p));
-LOCAL int	isonum_733	__PR((unsigned char * p));
+LOCAL int	isonum_733	__PR((char * p));
 LOCAL int	parse_rr	__PR((unsigned char * pnt, int len, int cont_flag));
+LOCAL void	find_rr		__PR((struct iso_directory_record * idr, Uchar **pntp, int *lenp));
 LOCAL int	dump_rr		__PR((struct iso_directory_record * idr));
 LOCAL void	check_tree	__PR((off_t file_addr, int file_size, off_t parent_addr));
+#if 0
 LOCAL void	check_path_tables __PR((int typel_extent, int typem_extent, int path_table_size));
+#endif
 LOCAL void	usage		__PR((int excode));
 EXPORT int	main		__PR((int argc, char *argv[]));
 
@@ -150,12 +106,8 @@ isonum_723(p)
 {
 #if 0
 	if (p[0] != p[3] || p[1] != p[2]) {
-#ifdef	USE_LIBSCHILY
-		comerrno(EX_BAD, "invalid format 7.2.3 number\n");
-#else
 		fprintf(stderr, "invalid format 7.2.3 number\n");
 		exit(1);
-#endif
 	}
 #endif
 	return (isonum_721(p));
@@ -178,6 +130,7 @@ isonum_731(p)
 		| ((p[3] & 0xff) << 24));
 }
 
+#if 0
 LOCAL int
 isonum_722(p)
 	char	*p;
@@ -185,6 +138,7 @@ isonum_722(p)
 	return ((p[1] & 0xff)
 		| ((p[0] & 0xff) << 8));
 }
+#endif
 
 LOCAL int
 isonum_732(p)
@@ -198,9 +152,9 @@ isonum_732(p)
 
 LOCAL int
 isonum_733(p)
-	unsigned char	*p;
+	char	*p;
 {
-	return (isonum_731((char *)p));
+	return (isonum_731(p));
 }
 
 char	lbuffer[1024];
@@ -234,7 +188,8 @@ parse_rr(pnt, len, cont_flag)
 	symlinkname[0] = 0;
 
 	ncount = 0;
-	flag1 = flag2 = 0;
+	flag1 = -1;
+	flag2 = 0;
 	while (len >= 4) {
 		if (ncount)
 			sprintf(lbuffer+iline, ",");
@@ -260,26 +215,30 @@ parse_rr(pnt, len, cont_flag)
 		}
 		ncount++;
 		if (pnt[0] == 'R' && pnt[1] == 'R') flag1 = pnt[4] & 0xff;
-		if (strncmp((char *)pnt, "PX", 2) == 0) flag2 |= 1;
-		if (strncmp((char *)pnt, "PN", 2) == 0) flag2 |= 2;
-		if (strncmp((char *)pnt, "SL", 2) == 0) flag2 |= 4;
-		if (strncmp((char *)pnt, "NM", 2) == 0) flag2 |= 8;
-		if (strncmp((char *)pnt, "CL", 2) == 0) flag2 |= 16;
-		if (strncmp((char *)pnt, "PL", 2) == 0) flag2 |= 32;
-		if (strncmp((char *)pnt, "RE", 2) == 0) flag2 |= 64;
-		if (strncmp((char *)pnt, "TF", 2) == 0) flag2 |= 128;
+		if (strncmp((char *)pnt, "PX", 2) == 0) flag2 |= RR_FLAG_PX;
+		if (strncmp((char *)pnt, "PN", 2) == 0) flag2 |= RR_FLAG_PN;
+		if (strncmp((char *)pnt, "SL", 2) == 0) flag2 |= RR_FLAG_SL;
+		if (strncmp((char *)pnt, "NM", 2) == 0) flag2 |= RR_FLAG_NM;
+		if (strncmp((char *)pnt, "CL", 2) == 0) flag2 |= RR_FLAG_CL;
+		if (strncmp((char *)pnt, "PL", 2) == 0) flag2 |= RR_FLAG_PL;
+		if (strncmp((char *)pnt, "RE", 2) == 0) flag2 |= RR_FLAG_RE;
+		if (strncmp((char *)pnt, "TF", 2) == 0) flag2 |= RR_FLAG_TF;
 
 		if (strncmp((char *)pnt, "CE", 2) == 0) {
-			cont_extent = (off_t)isonum_733(pnt+4);
-			cont_offset = isonum_733(pnt+12);
-			cont_size = isonum_733(pnt+20);
+			cont_extent = (off_t)isonum_733((char *)pnt+4);
+			cont_offset = isonum_733((char *)pnt+12);
+			cont_size = isonum_733((char *)pnt+20);
 			sprintf(lbuffer+iline, "=[%x,%x,%d]",
 					(int)cont_extent, cont_offset, cont_size);
 			iline += strlen(lbuffer + iline);
 		}
 
+		if (strncmp((char *)pnt, "ST", 2) == 0) {		/* Terminate SUSP */
+			break;
+		}
+
 		if (strncmp((char *)pnt, "PL", 2) == 0 || strncmp((char *)pnt, "CL", 2) == 0) {
-			extent = isonum_733(pnt+4);
+			extent = isonum_733((char *)pnt+4);
 			sprintf(lbuffer+iline, "=%x", extent);
 			iline += strlen(lbuffer + iline);
 			if (extent == 0)
@@ -336,22 +295,22 @@ parse_rr(pnt, len, cont_flag)
 
 		len -= pnt[2];
 		pnt += pnt[2];
-		if (len <= 3 && cont_extent) {
-			unsigned char sector[2048];
+	}
+	if (cont_extent) {
+		unsigned char sector[2048];
 #ifdef	USE_SCG
-			readsecs(cont_extent * blocksize / 2048, sector, ISO_BLOCKS(sizeof (sector)));
+		readsecs(cont_extent * blocksize / 2048, sector, ISO_BLOCKS(sizeof (sector)));
 #else
-			lseek(fileno(infile), cont_extent * blocksize, SEEK_SET);
-			read(fileno(infile), sector, sizeof (sector));
+		lseek(fileno(infile), cont_extent * blocksize, SEEK_SET);
+		read(fileno(infile), sector, sizeof (sector));
 #endif
-			flag2 |= parse_rr(&sector[cont_offset], cont_size, 1);
-		}
+		flag2 |= parse_rr(&sector[cont_offset], cont_size, 1);
 	}
 	if (ncount) {
 		sprintf(lbuffer+iline, "]");
 		iline += strlen(lbuffer + iline);
 	}
-	if (!cont_flag && flag1 && flag1 != flag2) {
+	if (!cont_flag && flag1 != -1 && flag1 != (flag2 & 0xFF)) {
 		sprintf(lbuffer+iline, "Flag %x != %x", flag1, flag2);
 		rr_goof++;
 		iline += strlen(lbuffer + iline);
@@ -359,27 +318,50 @@ parse_rr(pnt, len, cont_flag)
 	return (flag2);
 }
 
+LOCAL void
+find_rr(idr, pntp, lenp)
+	struct iso_directory_record *idr;
+	Uchar	**pntp;
+	int	*lenp;
+{
+	struct iso_xa_dir_record *xadp;
+	int len;
+	unsigned char * pnt;
+
+	len = idr->length[0] & 0xff;
+	len -= offsetof(struct iso_directory_record, name[0]);
+	len -= idr->name_len[0];
+
+	pnt = (unsigned char *) idr;
+	pnt += offsetof(struct iso_directory_record, name[0]);
+	pnt += idr->name_len[0];
+	if ((idr->name_len[0] & 1) == 0) {
+		pnt++;
+		len--;
+	}
+	if (len >= 14) {
+		xadp = (struct iso_xa_dir_record *)pnt;
+
+		if (xadp->signature[0] == 'X' && xadp->signature[1] == 'A' &&
+		    xadp->reserved[0] == '\0') {
+			len -= 14;
+			pnt += 14;
+		}
+	}
+	*pntp = pnt;
+	*lenp = len;
+}
+
 LOCAL int
 dump_rr(idr)
 	struct iso_directory_record *idr;
 {
 	int len;
-	char * pnt;
-
-	len = idr->length[0] & 0xff;
-	len -= offsetof(struct iso_directory_record, name[0]);
-	len -= idr->name_len[0];
-	pnt = (char *) idr;
-	pnt += offsetof(struct iso_directory_record, name[0]);
-	pnt += idr->name_len[0];
-
-	if ((idr->name_len[0] & 1) == 0) {
-		pnt++;
-		len--;
-	}
+	unsigned char * pnt;
 
 	rr_goof = 0;
-	parse_rr((unsigned char *)pnt, len, 0);
+	find_rr(idr, &pnt, &len);
+	parse_rr(pnt, len, 0);
 	return (rr_goof);
 }
 
@@ -462,7 +444,8 @@ check_tree(file_addr, file_size, parent_addr)
 				sprintf(&lbuffer[iline], ".             ");
 				iline += strlen(lbuffer + iline);
 				rflag = 0;
-				if (orig_file_addr != (off_t)(isonum_733(idr->extent) + isonum_711((char *) idr->ext_attr_length))) {
+				if (orig_file_addr != (off_t)(isonum_733(idr->extent) +
+							isonum_711((char *) idr->ext_attr_length))) {
 					sprintf(&lbuffer[iline], "***** Directory has null extent.");
 					goof++;
 					iline += strlen(lbuffer + iline);
@@ -476,7 +459,8 @@ check_tree(file_addr, file_size, parent_addr)
 				sprintf(&lbuffer[iline], "..            ");
 				iline += strlen(lbuffer + iline);
 				rflag = 0;
-				if (parent_file_addr != (off_t)(isonum_733(idr->extent) + isonum_711((char *) idr->ext_attr_length))) {
+				if (parent_file_addr != (off_t)(isonum_733(idr->extent) +
+							isonum_711((char *) idr->ext_attr_length))) {
 					sprintf(&lbuffer[iline], "***** Directory has null extent.");
 					goof++;
 					iline += strlen(lbuffer + iline);
@@ -582,6 +566,7 @@ struct path_table_info {
 	unsigned short	parent;
 };
 
+#if 0
 LOCAL void
 check_path_tables(typel_extent, typem_extent, path_table_size)
 	int	typel_extent;
@@ -659,7 +644,10 @@ check_path_tables(typel_extent, typem_extent, path_table_size)
 		}
 		printf("%4.4d %4.4d %8.8x %s\n", count++, idx, extent, name);
 	}
+	free(typel);
+	free(typem);
 }
+#endif
 
 LOCAL void
 usage(excode)
@@ -688,7 +676,7 @@ main(argc, argv)
 	BOOL	help = FALSE;
 	BOOL	prvers = FALSE;
 	char	*filename = NULL;
-	char	*devname = NULL;
+	char	*sdevname = NULL;
 	off_t	file_addr;
 	int	file_size;
 	struct iso_primary_descriptor	ipd;
@@ -702,20 +690,21 @@ main(argc, argv)
 	cac = argc - 1;
 	cav = argv + 1;
 	if (getallargs(&cac, &cav, opts, &help, &help, &prvers,
-			&filename, &devname) < 0) {
+			&filename, &sdevname) < 0) {
 		errmsgno(EX_BAD, "Bad Option: '%s'\n", cav[0]);
 		usage(EX_BAD);
 	}
 	if (help)
 		usage(0);
 	if (prvers) {
-		printf("isovfy %s (%s-%s-%s)\n", "2.01",
+		printf("isovfy %s (%s-%s-%s) Copyright (C) 1993-1999 Eric Youngdale (C) 1999-2010 Jrg Schilling\n",
+					VERSION,
 					HOST_CPU, HOST_VENDOR, HOST_OS);
 		exit(0);
 	}
 	cac = argc - 1;
 	cav = argv + 1;
-	if (filename == NULL && devname == NULL) {
+	if (filename == NULL && sdevname == NULL) {
 		if (getfiles(&cac, &cav, opts) != 0) {
 			filename = cav[0];
 			cac--, cav++;
@@ -725,27 +714,23 @@ main(argc, argv)
 		errmsgno(EX_BAD, "Bad Argument: '%s'\n", cav[0]);
 		usage(EX_BAD);
 	}
-	if (filename != NULL && devname != NULL) {
+	if (filename != NULL && sdevname != NULL) {
 		errmsgno(EX_BAD, "Only one of -i or dev= allowed\n");
 		usage(EX_BAD);
 	}
 #ifdef	USE_SCG
-	if (filename == NULL && devname == NULL)
-		cdr_defaults(&devname, NULL, NULL, NULL);
+	if (filename == NULL && sdevname == NULL)
+		cdr_defaults(&sdevname, NULL, NULL, NULL, NULL);
 #endif
-	if (filename == NULL && devname == NULL) {
-#ifdef	USE_LIBSCHILY
-		errmsgno(EX_BAD, "ISO-9660 image not specified\n");
-#else
+	if (filename == NULL && sdevname == NULL) {
 		fprintf(stderr, "ISO-9660 image not specified\n");
-#endif
 		usage(EX_BAD);
 	}
 
 	if (filename != NULL)
 		infile = fopen(filename, "rb");
 	else
-		filename = devname;
+		filename = sdevname;
 
 	if (infile != NULL) {
 		/* EMPTY */;
@@ -754,12 +739,8 @@ main(argc, argv)
 #else
 	} else {
 #endif
-#ifdef	USE_LIBSCHILY
-		comerr("Cannot open '%s'\n", filename);
-#else
 		fprintf(stderr, "Cannot open '%s'\n", filename);
 		exit(1);
-#endif
 	}
 
 
@@ -799,7 +780,8 @@ main(argc, argv)
 	check_path_tables(typel_extent, typem_extent, path_table_size);
 #endif
 
-	fclose(infile);
+	if (infile != NULL)
+		fclose(infile);
 
 	if (!ngoof)
 		printf("No errors found\n");
