@@ -1,10 +1,12 @@
-/* @(#)mac_label.c	1.6 02/05/20 joerg, Copyright 1997, 1998, 1999, 2000 James Pearson */
+/* @(#)mac_label.c	1.18 09/11/25 joerg, Copyright 1997-2000 James Pearson, Copyright 2004-2009 J. Schilling */
+#include <schily/mconfig.h>
 #ifndef lint
-static	char sccsid[] =
-	"@(#)mac_label.c	1.6 02/05/20 joerg, Copyright 1997, 1998, 1999, 2000 James Pearson";
+static	UConst char sccsid[] =
+	"@(#)mac_label.c	1.18 09/11/25 joerg, Copyright 1997-2000 James Pearson, Copyright 2004-2009 J. Schilling";
 #endif
 /*
  *      Copyright (c) 1997, 1998, 1999, 2000 James Pearson
+ *	Copyright (c) 2004-2009 J. Schilling
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -44,10 +46,10 @@ static	char sccsid[] =
 
 /* PREP_BOOT Troy Benjegerdes 2/4/99 */
 
-#include <mconfig.h>
 #include "mkisofs.h"
-#include <mac_label.h>
+#include "mac_label.h"
 #include "apple.h"
+#include <schily/schily.h>
 
 #ifdef PREP_BOOT
 void	gen_prepboot_label	__PR((unsigned char *ml));
@@ -62,44 +64,92 @@ gen_prepboot_label(ml)
 	unsigned char	*ml;
 {
 	struct directory_entry *de;
-	int		i,
-			block,
-			size;
+	int		i = 0;
+	int		block;
+	int		size;
 	MacLabel	*mac_label = (MacLabel *) ml;
 
 	if (verbose > 1) {
 		fprintf(stderr, "Creating %d PReP boot partition(s)\n",
-								use_prep_boot);
+						use_prep_boot + use_chrp_boot);
 	}
 	mac_label->fdiskMagic[0] = fdiskMagic0;
 	mac_label->fdiskMagic[1] = fdiskMagic1;
 
-	for (i = 0; i < use_prep_boot; i++) {
-		de = search_tree_file(root, prep_boot_image[i]);
+	if (use_chrp_boot) {
+		fprintf(stderr, "CHRP boot partition 1\n");
+
+		mac_label->image[i].boot = 0x80;
+
+		mac_label->image[i].CHSstart[0] = 0xff;
+		mac_label->image[i].CHSstart[1] = 0xff;
+		mac_label->image[i].CHSstart[2] = 0xff;
+
+		mac_label->image[i].type = chrpPartType;	/* 0x96 */
+
+		mac_label->image[i].CHSend[0] = 0xff;
+		mac_label->image[i].CHSend[1] = 0xff;
+		mac_label->image[i].CHSend[2] = 0xff;
+
+		mac_label->image[i].startSect[0] = 0;
+		mac_label->image[i].startSect[1] = 0;
+		mac_label->image[i].startSect[2] = 0;
+		mac_label->image[i].startSect[3] = 0;
+
+		size = (last_extent - session_start) * 2048 / 512;
+		mac_label->image[i].size[0] = size & 0xff;
+		mac_label->image[i].size[1] = (size >> 8) & 0xff;
+		mac_label->image[i].size[2] = (size >> 16) & 0xff;
+		mac_label->image[i].size[3] = (size >> 24) & 0xff;
+
+		i++;
+	}
+
+	for (; i < use_prep_boot + use_chrp_boot; i++) {
+		de = search_tree_file(root, prep_boot_image[i - use_chrp_boot]);
 		if (!de) {
-			fprintf(stderr,
-				"Uh oh, I cant find the boot image \"%s\"!\n",
-				prep_boot_image[i]);
-			exit(1);
+			ex_boot_enoent("image",
+				prep_boot_image[i - use_chrp_boot]);
+			/* NOTREACHED */
 		}
 		/* get size and block in 512-byte blocks */
 		block = get_733(de->isorec.extent) * 2048 / 512;
 		size = get_733(de->isorec.size) / 512 + 1;
 		fprintf(stderr, "PReP boot partition %d is \"%s\"\n",
-			i + 1, prep_boot_image[i]);
+			i + 1, prep_boot_image[i - use_chrp_boot]);
+
+		mac_label->image[i].boot = 0x80;
+
+		mac_label->image[i].CHSstart[0] = 0xff;
+		mac_label->image[i].CHSstart[1] = 0xff;
+		mac_label->image[i].CHSstart[2] = 0xff;
+
+		mac_label->image[i].type = prepPartType;	/* 0x41 */
+
+		mac_label->image[i].CHSend[0] = 0xff;
+		mac_label->image[i].CHSend[1] = 0xff;
+		mac_label->image[i].CHSend[2] = 0xff;
 
 		/* deal with  endianess */
-		mac_label->image[i].type = prepPartType;	/* 0x41 */
 		mac_label->image[i].startSect[0] = block & 0xff;
 		mac_label->image[i].startSect[1] = (block >> 8) & 0xff;
 		mac_label->image[i].startSect[2] = (block >> 16) & 0xff;
 		mac_label->image[i].startSect[3] = (block >> 24) & 0xff;
+
 		mac_label->image[i].size[0] = size & 0xff;
 		mac_label->image[i].size[1] = (size >> 8) & 0xff;
 		mac_label->image[i].size[2] = (size >> 16) & 0xff;
 		mac_label->image[i].size[3] = (size >> 24) & 0xff;
 	}
+	for (; i < 4; i++) {
+		mac_label->image[i].CHSstart[0] = 0xff;
+		mac_label->image[i].CHSstart[1] = 0xff;
+		mac_label->image[i].CHSstart[2] = 0xff;
 
+		mac_label->image[i].CHSend[0] = 0xff;
+		mac_label->image[i].CHSend[1] = 0xff;
+		mac_label->image[i].CHSend[2] = 0xff;
+	}
 }
 
 #endif	/* PREP_BOOT */
@@ -111,7 +161,7 @@ gen_mac_label(mac_boot)
 	FILE		*fp;
 	MacLabel	*mac_label;
 	MacPart		*mac_part;
-	char		*buffer = (char *) hce->hfs_map;
+	char		*buffer = (char *)hce->hfs_map;
 	int		block_size;
 	int		have_hfs_boot = 0;
 	char		tmp[SECTOR_SIZE];
@@ -138,10 +188,10 @@ gen_mac_label(mac_boot)
 			return (-1);
 		}
 		/* check we have a bootable partition */
-		mac_part = (MacPart *) (tmp + HFS_BLOCKSZ);
+		mac_part = (MacPart *)(tmp + HFS_BLOCKSZ);
 
 		if (!(IS_MAC_PART(mac_part) &&
-		   !strncmp((char *) mac_part->pmPartType, pmPartType_2, 12))) {
+		    strncmp((char *)mac_part->pmPartType, pmPartType_2, 12) == 0)) {
 			sprintf(hce->error, "%s is not a HFS boot file",
 								mac_boot->name);
 			return (-1);
@@ -162,7 +212,7 @@ gen_mac_label(mac_boot)
 		fclose(fp);
 
 		/* check boot block is valid */
-		if (d_getw((unsigned char *) hce->hfs_hdr) != HFS_BB_SIGWORD) {
+		if (d_getw((unsigned char *)hce->hfs_hdr) != HFS_BB_SIGWORD) {
 			sprintf(hce->error,
 				"%s does not contain a valid boot block",
 								mac_boot->name);
@@ -183,7 +233,7 @@ gen_mac_label(mac_boot)
 		mpm[mpc].size = ISO_BLOCKS(mac_boot->size);
 
 		mpm[mpc].ntype = PM2;
-		mpm[mpc].type = (char *) mac_part->pmPartType;
+		mpm[mpc].type = (char *)mac_part->pmPartType;
 		mpm[mpc].start = mac_boot->extent = last_extent;
 		mpm[mpc].name = 0;
 
@@ -215,61 +265,61 @@ gen_mac_label(mac_boot)
 	block_size = have_hfs_boot ? SECTOR_SIZE : HFS_BLOCKSZ;
 
 	/* create the CD label */
-	mac_label = (MacLabel *) buffer;
+	mac_label = (MacLabel *)buffer;
 	mac_label->sbSig[0] = 'E';
 	mac_label->sbSig[1] = 'R';
-	set_722((char *) mac_label->sbBlkSize, block_size);
-	set_732((char *) mac_label->sbBlkCount,
+	set_722((char *)mac_label->sbBlkSize, block_size);
+	set_732((char *)mac_label->sbBlkCount,
 				last_extent * (SECTOR_SIZE / block_size));
-	set_722((char *) mac_label->sbDevType, 1);
-	set_722((char *) mac_label->sbDevId, 1);
+	set_722((char *)mac_label->sbDevType, 1);
+	set_722((char *)mac_label->sbDevId, 1);
 
 	/* create the partition map entry */
-	mac_part = (MacPart *) (buffer + block_size);
+	mac_part = (MacPart *)(buffer + block_size);
 	mac_part->pmSig[0] = 'P';
 	mac_part->pmSig[1] = 'M';
-	set_732((char *) mac_part->pmMapBlkCnt, mpc + 1);
-	set_732((char *) mac_part->pmPyPartStart, 1);
-	set_732((char *) mac_part->pmPartBlkCnt, mpc + 1);
-	strncpy((char *) mac_part->pmPartName, "Apple",
-						sizeof(mac_part->pmPartName));
-	strncpy((char *) mac_part->pmPartType, "Apple_partition_map",
-						sizeof(mac_part->pmPartType));
-	set_732((char *) mac_part->pmLgDataStart, 0);
-	set_732((char *) mac_part->pmDataCnt, mpc + 1);
-	set_732((char *) mac_part->pmPartStatus, PM_STAT_DEFAULT);
+	set_732((char *)mac_part->pmMapBlkCnt, mpc + 1);
+	set_732((char *)mac_part->pmPyPartStart, 1);
+	set_732((char *)mac_part->pmPartBlkCnt, mpc + 1);
+	strncpy((char *)mac_part->pmPartName, "Apple",
+						sizeof (mac_part->pmPartName));
+	strncpy((char *)mac_part->pmPartType, "Apple_partition_map",
+						sizeof (mac_part->pmPartType));
+	set_732((char *)mac_part->pmLgDataStart, 0);
+	set_732((char *)mac_part->pmDataCnt, mpc + 1);
+	set_732((char *)mac_part->pmPartStatus, PM_STAT_DEFAULT);
 
 	/* create partition map entries for our partitions */
 	for (i = 0; i < mpc; i++) {
-		mac_part = (MacPart *) (buffer + (i + 2) * block_size);
+		mac_part = (MacPart *)(buffer + (i + 2) * block_size);
 		if (mpm[i].ntype == PM2) {
 			/* get driver label and patch it */
-			memcpy((char *) mac_label, tmp, HFS_BLOCKSZ);
-			set_732((char *) mac_label->sbBlkCount,
+			memcpy((char *)mac_label, tmp, HFS_BLOCKSZ);
+			set_732((char *)mac_label->sbBlkCount,
 				last_extent * (SECTOR_SIZE / block_size));
-			set_732((char *) mac_label->ddBlock,
+			set_732((char *)mac_label->ddBlock,
 				(mpm[i].start) * (SECTOR_SIZE / block_size));
-			memcpy((char *) mac_part, tmp + HFS_BLOCKSZ,
+			memcpy((char *)mac_part, tmp + HFS_BLOCKSZ,
 								HFS_BLOCKSZ);
-			set_732((char *) mac_part->pmMapBlkCnt, mpc + 1);
-			set_732((char *) mac_part->pmPyPartStart,
+			set_732((char *)mac_part->pmMapBlkCnt, mpc + 1);
+			set_732((char *)mac_part->pmPyPartStart,
 				(mpm[i].start) * (SECTOR_SIZE / block_size));
 		} else {
 			mac_part->pmSig[0] = 'P';
 			mac_part->pmSig[1] = 'M';
-			set_732((char *) mac_part->pmMapBlkCnt, mpc + 1);
-			set_732((char *) mac_part->pmPyPartStart,
+			set_732((char *)mac_part->pmMapBlkCnt, mpc + 1);
+			set_732((char *)mac_part->pmPyPartStart,
 				mpm[i].start * (SECTOR_SIZE / HFS_BLOCKSZ));
-			set_732((char *) mac_part->pmPartBlkCnt,
+			set_732((char *)mac_part->pmPartBlkCnt,
 				mpm[i].size * (SECTOR_SIZE / HFS_BLOCKSZ));
-			strncpy((char *) mac_part->pmPartName, mpm[i].name,
-				sizeof(mac_part->pmPartName));
-			strncpy((char *) mac_part->pmPartType, mpm[i].type,
-				sizeof(mac_part->pmPartType));
-			set_732((char *) mac_part->pmLgDataStart, 0);
-			set_732((char *) mac_part->pmDataCnt,
+			strncpy((char *)mac_part->pmPartName, mpm[i].name,
+				sizeof (mac_part->pmPartName));
+			strncpy((char *)mac_part->pmPartType, mpm[i].type,
+				sizeof (mac_part->pmPartType));
+			set_732((char *)mac_part->pmLgDataStart, 0);
+			set_732((char *)mac_part->pmDataCnt,
 				mpm[i].size * (SECTOR_SIZE / HFS_BLOCKSZ));
-			set_732((char *) mac_part->pmPartStatus,
+			set_732((char *)mac_part->pmPartStatus,
 				PM_STAT_DEFAULT);
 		}
 	}
@@ -279,50 +329,50 @@ gen_mac_label(mac_boot)
 		if (mpc < 3) {	/* don't have to interleave with 2048 table */
 			mac_part->pmSig[0] = 'P';
 			mac_part->pmSig[1] = 'M';
-			set_732((char *) mac_part->pmMapBlkCnt, mpc + 1);
-			set_732((char *) mac_part->pmPyPartStart, 1);
-			set_732((char *) mac_part->pmPartBlkCnt, mpc + 1);
-			strncpy((char *) mac_part->pmPartName, "Apple",
-					sizeof(mac_part->pmPartName));
-			strncpy((char *) mac_part->pmPartType,
+			set_732((char *)mac_part->pmMapBlkCnt, mpc + 1);
+			set_732((char *)mac_part->pmPyPartStart, 1);
+			set_732((char *)mac_part->pmPartBlkCnt, mpc + 1);
+			strncpy((char *)mac_part->pmPartName, "Apple",
+					sizeof (mac_part->pmPartName));
+			strncpy((char *)mac_part->pmPartType,
 					"Apple_partition_map",
-					sizeof(mac_part->pmPartType));
-			set_732((char *) mac_part->pmLgDataStart, 0);
-			set_732((char *) mac_part->pmDataCnt, mpc + 1);
-			set_732((char *) mac_part->pmPartStatus,
+					sizeof (mac_part->pmPartType));
+			set_732((char *)mac_part->pmLgDataStart, 0);
+			set_732((char *)mac_part->pmDataCnt, mpc + 1);
+			set_732((char *)mac_part->pmPartStatus,
 							PM_STAT_DEFAULT);
 			mac_part++;	/* +HFS_BLOCKSZ */
 		}
 		for (i = 0; i < mpc; i++, mac_part++) {
-			if (mac_part == (MacPart *) (buffer + SECTOR_SIZE))
-				mac_part++;	/* jump over 2048 partition
-						   entry */
+			if (mac_part == (MacPart *)(buffer + SECTOR_SIZE))
+				mac_part++;	/* jump over 2048 partition */
+						/* entry */
 			if (mpm[i].ntype == PM2) {
-				memcpy((char *) mac_part, tmp + HFS_BLOCKSZ * 2,
+				memcpy((char *)mac_part, tmp + HFS_BLOCKSZ * 2,
 							HFS_BLOCKSZ);
 				if (!IS_MAC_PART(mac_part)) {
 					mac_part--;
 					continue;
 				}
-				set_732((char *) mac_part->pmMapBlkCnt, mpc+1);
-				set_732((char *) mac_part->pmPyPartStart,
+				set_732((char *)mac_part->pmMapBlkCnt, mpc+1);
+				set_732((char *)mac_part->pmPyPartStart,
 				    mpm[i].start * (SECTOR_SIZE / HFS_BLOCKSZ));
 			} else {
 				mac_part->pmSig[0] = 'P';
 				mac_part->pmSig[1] = 'M';
-				set_732((char *) mac_part->pmMapBlkCnt, mpc+1);
-				set_732((char *) mac_part->pmPyPartStart,
+				set_732((char *)mac_part->pmMapBlkCnt, mpc+1);
+				set_732((char *)mac_part->pmPyPartStart,
 				    mpm[i].start * (SECTOR_SIZE / HFS_BLOCKSZ));
-				set_732((char *) mac_part->pmPartBlkCnt,
+				set_732((char *)mac_part->pmPartBlkCnt,
 				    mpm[i].size * (SECTOR_SIZE / HFS_BLOCKSZ));
-				strncpy((char *) mac_part->pmPartName,
-				    mpm[i].name, sizeof(mac_part->pmPartName));
-				strncpy((char *) mac_part->pmPartType,
-				    mpm[i].type, sizeof(mac_part->pmPartType));
-				set_732((char *) mac_part->pmLgDataStart, 0);
-				set_732((char *) mac_part->pmDataCnt,
+				strncpy((char *)mac_part->pmPartName,
+				    mpm[i].name, sizeof (mac_part->pmPartName));
+				strncpy((char *)mac_part->pmPartType,
+				    mpm[i].type, sizeof (mac_part->pmPartType));
+				set_732((char *)mac_part->pmLgDataStart, 0);
+				set_732((char *)mac_part->pmDataCnt,
 				    mpm[i].size * (SECTOR_SIZE / HFS_BLOCKSZ));
-				set_732((char *) mac_part->pmPartStatus,
+				set_732((char *)mac_part->pmPartStatus,
 							PM_STAT_DEFAULT);
 			}
 		}
