@@ -1,45 +1,45 @@
-/* @(#)name.c	1.25 02/12/25 joerg */
+/* @(#)name.c	1.37 09/07/10 joerg */
+#include <schily/mconfig.h>
 #ifndef lint
-static	char sccsid[] =
-	"@(#)name.c	1.25 02/12/25 joerg";
+static	UConst char sccsid[] =
+	"@(#)name.c	1.37 09/07/10 joerg";
 
 #endif
 /*
  * File name.c - map full Unix file names to unique 8.3 names that
  * would be valid on DOS.
  *
+ *
+ * Written by Eric Youngdale (1993).
+ * Almost totally rewritten by J. Schilling (2000).
+ *
+ * Copyright 1993 Yggdrasil Computing, Incorporated
+ * Copyright (c) 1999,2000-2009 J. Schilling
+ *
+ * This program is free software; you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License as published by
+ * the Free Software Foundation; either version 2, or (at your option)
+ * any later version.
+ *
+ * This program is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU General Public License for more details.
+ *
+ * You should have received a copy of the GNU General Public License
+ * along with this program; if not, write to the Free Software
+ * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+ */
 
-   Written by Eric Youngdale (1993).
-   Almost totally rewritten by J. Schilling (2000).
-
-   Copyright 1993 Yggdrasil Computing, Incorporated
-   Copyright (c) 1999,2000 J. Schilling
-
-
-   This program is free software; you can redistribute it and/or modify
-   it under the terms of the GNU General Public License as published by
-   the Free Software Foundation; either version 2, or (at your option)
-   any later version.
-
-   This program is distributed in the hope that it will be useful,
-   but WITHOUT ANY WARRANTY; without even the implied warranty of
-   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-   GNU General Public License for more details.
-
-   You should have received a copy of the GNU General Public License
-   along with this program; if not, write to the Free Software
-   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.  */
-
-#include <mconfig.h>
 #include "mkisofs.h"
-#include <standard.h>
-#include <schily.h>
-#include <ctype.h>
+#include <schily/standard.h>
+#include <schily/schily.h>
+#include <schily/ctype.h>
 
 void	iso9660_check		__PR((struct iso_directory_record *idr,	struct directory_entry *ndr));
 int	iso9660_file_length	__PR((const char *name,
-			                struct directory_entry *sresult,
-			                int dirflag));
+					struct directory_entry *sresult,
+					int dirflag));
 
 void
 iso9660_check(idr, ndr)
@@ -67,7 +67,7 @@ iso9660_check(idr, ndr)
 	if ((np = strrchr(ndr->isorec.name, ';')) != NULL) {
 		*np = '\0';		/* Strip off new version # */
 	}
-	if (strcmp(idr->name, ndr->isorec.name)) {
+	if (strcmp(idr->name, ndr->isorec.name) != 0) {
 		if (p)
 			*p = ';';	/* Restore old version # */
 		if (np)
@@ -124,7 +124,9 @@ iso9660_file_length(name, sresult, dirflag)
 	int		ochars_before_dot;
 	int		seen_dot = 0;
 	int		seen_semic = 0;
+#ifdef		Eric_code_does_not_work
 	int		tildes = 0;
+#endif
 
 	result = sresult->isorec.name;
 
@@ -137,7 +139,7 @@ iso9660_file_length(name, sresult, dirflag)
 	 */
 	if (strcmp(name, ".") == 0) {
 		*result = 0;
-		return 1;
+		return (1);
 	}
 	/*
 	 * For the '..' entry, generate the correct record, and return 1
@@ -146,7 +148,7 @@ iso9660_file_length(name, sresult, dirflag)
 	if (strcmp(name, "..") == 0) {
 		*result++ = 1;
 		*result++ = 0;
-		return 1;
+		return (1);
 	}
 	/*
 	 * Now scan the directory one character at a time, and figure out
@@ -202,7 +204,7 @@ iso9660_file_length(name, sresult, dirflag)
 				before_dot = iso9660_namelen/2;
 				after_dot = ochars_after_dot;
 			} else {
-				before_dot -= ochars_after_dot;/* dot counts */
+				before_dot -= ochars_after_dot; /* dot counts */
 				after_dot = ochars_after_dot;
 			}
 		}
@@ -253,7 +255,7 @@ iso9660_file_length(name, sresult, dirflag)
 			 * Check if a valid version number follows.
 			 * The maximum valid version number is 32767.
 			 */
-			for (c = 1, cp = (char *)&pnt[1]; c < 6 && *cp; c++,cp++) {
+			for (c = 1, cp = (char *)&pnt[1]; c < 6 && *cp; c++, cp++) {
 				if (*cp < '0' || *cp > '9')
 					break;
 			}
@@ -335,13 +337,37 @@ iso9660_file_length(name, sresult, dirflag)
 			}
 		} else {
 			if ((seen_dot && (chars_after_dot < after_dot) &&
-						++chars_after_dot)
-			    || (!seen_dot && (chars_before_dot < before_dot) &&
+						++chars_after_dot) ||
+			    (!seen_dot && (chars_before_dot < before_dot) &&
 			    ++chars_before_dot)) {
 
 				c = *pnt;
 				if (c & 0x80) {
-					c = '_';
+					/*
+					 * We allow 8 bit chars if -iso-level
+					 * is at least 4
+					 *
+					 * XXX We should check if the output
+					 * XXX character set is a 7 Bit ASCI
+					 * extension.
+					 */
+					if (iso9660_level >= 4) {
+						size_t	flen = 1;
+						size_t	tlen = 1;
+
+						/*
+						 * XXX This currently only works for
+						 * XXX non iconv() based locales.
+						 */
+						if (in_nls->sic_cd2uni == NULL) {
+							conv_charset(
+							    (Uchar *)&c, &tlen,
+							    (Uchar *)&c, &flen,
+							    in_nls, out_nls);
+						}
+					} else {
+						c = '_';
+					}
 				} else if (!allow_lowercase) {
 					c = islower((unsigned char)c) ?
 						toupper((unsigned char)c) : c;
@@ -350,6 +376,8 @@ iso9660_file_length(name, sresult, dirflag)
 					/*
 					 * Here we allow a more relaxed syntax.
 					 */
+					if (c == '/')
+						c = '_';
 					*result++ = c;
 				} else switch (c) {
 					/*
@@ -372,8 +400,8 @@ iso9660_file_length(name, sresult, dirflag)
 				/* separators */
 				case '+':
 				case '=':
-				case '%': /* not legal DOS
-					   filename */
+				case '%': /* not legal DOS */
+						/* filename */
 				case ':':
 				case ';': /* already handled */
 				case '.': /* already handled */
@@ -464,8 +492,9 @@ iso9660_file_length(name, sresult, dirflag)
 	 * In case of name conflicts, this is what would end up being used as
 	 * the 'extension'.
 	 */
+#ifdef		Eric_code_does_not_work
 	if (tildes == 2) {
-		int             prio1 = 0;
+		int	prio1 = 0;
 
 		pnt = name;
 		while (*pnt && *pnt != '~') {
@@ -480,6 +509,7 @@ iso9660_file_length(name, sresult, dirflag)
 		}
 		priority = prio1;
 	}
+#endif
 	/*
 	 * If this is not a directory, force a '.' in case we haven't seen one,
 	 * and add a version number if we haven't seen one of those either.
